@@ -1,7 +1,7 @@
-#'@export 
+#'@export
 validate::as_yaml
 
-#'@export 
+#'@export
 validate::export_yaml
 
 #'@export
@@ -10,7 +10,6 @@ validate::origin
 #'@export
 validate::`origin<-`
 
-
 #'@export
 validate::description
 
@@ -21,4 +20,19 @@ validate::"description<-"
 validate::label
 
 #'@export
-validate::`label<-`
\ No newline at end of file
+validate::`label<-`
+
+#'@export
+validate::do_by
+
+#'@export
+validate::mean_by
+
+#'@export
+validate::sum_by
+
+#'@export
+validate::max_by
+
+#'@export
+validate::min_by
